/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

void chain_init(rf_chain_idx_t* clust, rf_int_t all)
{
    clust->chain_offset = 0;
    clust->cluster_offset = 0;
    clust->sector_offset = 0;
    clust->byte_offset = 0;
    if (all) {
        clust->last_chain_index = 0;
        clust->chain[0].cluster = 0;
        clust->chain[0].number = 0;
    }
}

#if 0
void chain_offset_next_cluster(rf_chain_idx_t* clust)
{
    clust->sector_offset = 0;
    clust->cluster_offset++;
}
#endif

#if RF_OPT_USE_NTFS
rf_u32_t chain_get_last_clust(rf_chain_idx_t* clust)
{
    rf_u8_t i;

    if (clust->last_chain_index == 0)
        return (rf_s32_t)NULL_CLUSTER;

    for (i = clust->last_chain_index - 1; i != UCHAR_MAX; i--) {
        if (clust->chain[ i ].cluster != NULL_CLUSTER)
            return clust->chain[ i ].cluster;
    }
    return (rf_s32_t)NULL_CLUSTER;
}
#endif

/* õǰռ */
#if 0
rf_u32_t chain_offset_clust(rf_chain_idx_t* clust)
{
    rf_u32_t i;
    rf_u32_t offset_clust;

    offset_clust = 0;
    for (i = 0; i != clust->last_chain_index; i++)
        offset_clust += clust->chain[i].number;
    return offset_clust;
}
#endif

/* ǰƫɸ */
rf_int_t chain_offset_sector(rf_chain_idx_t* clust, rf_int_t sector_per_clst, rf_u32_t offset)
{
    if (clust->chain_offset > clust->last_chain_index)
        return RF_ENDFILE;
    while (offset) {
        clust->sector_offset++;
        if (clust->sector_offset == sector_per_clst) {
            clust->sector_offset = 0;
            clust->cluster_offset++;
            if (clust->chain[ clust->chain_offset ].number == clust->cluster_offset) {
                clust->cluster_offset = 0;
                clust->chain_offset++;
            }
            /* ƫƵβ chain_handle.index  chain_handle.last_index */
            if (clust->chain_offset > clust->last_chain_index)
                return RF_ENDFILE;
        }
        offset--;
    }
    return RF_ESUCCESS;
}

rf_int_t chain_back_offset_sector(rf_chain_idx_t* clust, rf_int_t sector_per_clst, rf_u32_t offset)
{
    while (offset) {
        if (clust->sector_offset == 0) {
            clust->sector_offset = sector_per_clst;
            if (clust->cluster_offset == 0) {
                RF_ASSERTE(("err: clust->chain_offset==0\n"), clust->chain_offset > 0);
                clust->chain_offset--;
                clust->cluster_offset = clust->chain[clust->chain_offset].number;
            }
            clust->cluster_offset--;
        }
        clust->sector_offset--;
        offset--;
    }
    return RF_ESUCCESS;
}

#if RF_OPT_USE_NTFS
rf_u32_t chain_count_clust(rf_chain_idx_t* clust)
{
    /*  */
    /*clust = clust;*/
    return 0;
}
#endif
