/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_COMMONX_H__
#define __RF_COMMONX_H__

enum {
    RF_TYPE_FAT_FS = 1,
    RF_TYPE_EXT_FS,
    RF_TYPE_NTFS_FS,
};

struct _rf_vol_comm_t {
    rf_u8_t     fs_type: 4;
    rf_u8_t     open_flag: 1;
    rf_u8_t     access_flag: 1;
#if RF_OPT_FS_READONLY == 0
    rf_u8_t     write_flag: 1;
#endif
#if RF_OPT_USE_MALLOC
    list_t      list;
#endif
#if RF_OPT_USE_VOLUME_CTX
    rf_void_t*   ctx;
#endif
    rf_handle_t dev_handle;
#if RF_OPT_USE_MALLOC == 1
    rf_u8_t* sector_buffer;
#else
    rf_u8_t sector_buffer[RF_OPT_MAX_SECTOR_SIZE];
#endif
};
typedef struct _rf_vol_comm_t rf_vol_comm_t, *rf_vol_comm_ref_t;

struct _rf_file_comm_t {
    /* 򿪻߹ر ־λ */
    rf_u8_t             open_flag: 1;
    /* 0ֻ 1д 2дȨ */
    rf_u8_t             access_flag: 2;
    /* 1 ļ־  */
    rf_u8_t             sector_buf_flag: 1;
    /* 1 ǰΪĿ¼ */
    rf_u8_t             root_dir_flag: 1;
    /* Ŀ¼ڷ */
    rf_void_t*          vol;
#if RF_OPT_USE_FILE_CTX
    rf_void_ref_t       ctx;
#endif
    /* ̬malloc fileʱ ʹ */
#if RF_OPT_USE_MALLOC
    list_t              list;
#endif
};
typedef struct _rf_file_comm_t rf_file_comm_t, *rf_file_comm_ref_t;

struct _rf_sector_buf_t {
    rf_void_t*  buffer;
    rf_u32_t    len;
};
typedef struct _rf_sector_buf_t rf_sector_buf_t, *rf_sector_buf_ref_t;

/* · */
#if RF_OPT_PATH_LOCAL == 1

#undef _RF_MAX_NAME
#if RF_OPT_USE_EXT2
#define _RF_MAX_NAME 256
#endif

#if RF_OPT_USE_NTFS
#undef _RF_MAX_NAME
#define _RF_MAX_NAME 512
#endif

#if RF_OPT_USE_NTFS
#undef _RF_MAX_NAME
#define _RF_MAX_NAME 512
#endif

#if RF_OPT_USE_FAT && RF_OPT_FAT_LONG_NAME
#undef _RF_MAX_NAME
#define _RF_MAX_NAME 520
#endif

#ifdef _RF_MAX_NAME
extern rf_char_t xdata g_open_name_buf[ _RF_MAX_NAME ];
extern rf_char_t xdata g_find_name_buf[ _RF_MAX_NAME ];

#define DEF_PATH_BUF
#define GET_PATH_BUF(buf)       { buf = g_open_name_buf; *((rf_u16_t *)buf) = 0; }
#define FREE_PATH_BUF(buf)

#define DEF_FIND_BUF
#define GET_FIND_BUF(buf)       { buf = g_find_name_buf; *((rf_u16_t *)buf) = 0; }
#define FREE_FIND_BUF(buf)
#endif

#elif RF_OPT_PATH_LOCAL == 2
/* ʹöջ Ϊ· */
#define DEF_PATH_BUF            rf_char_t g_open_name_buf[_RF_MAX_NAME];
#define GET_PATH_BUF(buf)       { buf = g_open_name_buf; *((rf_u16_t *)buf) = 0; }
#define FREE_PATH_BUF(buf)

#define DEF_FIND_BUF            rf_char_t g_find_name_buf[_RF_MAX_NAME];
#define GET_FIND_BUF(buf)       { buf = g_find_name_buf; *((rf_u16_t *)buf) = 0; }
#define FREE_FIND_BUF(buf)
#elif RF_OPT_PATH_LOCAL == 3
/* ʹmalloc Ϊ· */
#define DEF_PATH_BUF
#define GET_PATH_BUF(buf)       { buf = malloc(_RF_MAX_NAME); if (!buf) return RF_ENOMEM; *((rf_u16_t *)buf) = 0; }
#define FREE_PATH_BUF(buf)      rf_free(buf);

#define DEF_FIND_BUF
#define GET_FIND_BUF(buf)       { buf = malloc(_RF_MAX_NAME); if (!buf) return RF_ENOMEM; *((rf_u16_t *)buf) = 0; }
#define FREE_FIND_BUF(buf)      rf_free(buf);
#elif RF_OPT_PATH_LOCAL == 4
/* ʹû ļ ּ֧Сռ ʹraw-open ļ */
#define DEF_PATH_BUF
#define GET_PATH_BUF(buf)       { buf = NULL; }
#define FREE_PATH_BUF(buf)

#define DEF_FIND_BUF
#define GET_FIND_BUF(buf)       { buf = NULL; }
#define FREE_FIND_BUF(buf)
#else
#error Wrong RF_OPT_LOCAL_PATH setting
#endif

#define RF_VERSION_SHORT_STRING     __rf_mstrcat6__("rf_tag_ver= ", \
    __rf_mstring_ex__(__rf_mconcat7_ex__(RF_OPT_VERSION_MAJOR, ., \
     RF_OPT_VERSION_MINOR, ., RF_OPT_VERSION_ALTER, ., RF_OPT_VERSION_BUILD)), "_", RF_ARCH_VERSION_STRING, " by ", RF_COMPILER_VERSION_STRING)

#endif

