/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

rf_void_t list_init(list_t** head)
{
    *head = NULL;
}

rf_void_t list_add(list_t** head, list_t* item)
{
    list_t* l;

    RF_ASSERTE("item==NULL", item != NULL);
    if (*head == NULL) {
        *head = item;
        item->next = NULL;
        return;
    }
    l = (*head)->next;
    (*head)->next = item;
    item->next = l;
}

rf_int_t list_remove(list_t** head, list_t* item)
{
    list_t* l, *r;

    RF_ASSERTE("item==NULL", item != NULL);
    RF_ASSERTE("*head==NULL", *head != NULL);

    if (*head == item) {
        *head = item->next;
        return 0;
    }
    r = NULL;
    for (l = (*head)->next; l != NULL; l = l->next) {
        if (l == item) {
            if (r == NULL) {
                (*head)->next = l->next;
            } else {
                r->next = l->next;
            }
            l->next = NULL;
            return 0;
        }
        r = l;
    }
    return -1;
}

list_t* list_item_next(list_t* item)
{
    return item == NULL ? NULL : item->next;
}