/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

/*
ansiļ 
1ansiļ תΪunicode
2unicodeļ תΪansiļ
*/
rf_int_t path_get_sfn_lfn(const rf_char_t** path, rf_char_t* sfn, rf_wchar_t* lfn)
{
#if RF_OPT_FAT_LONG_NAME
    const rf_char_t* p;
    rf_u8_t b, cf;
    rf_u16_t w;
    rf_int_t i, ni, si, di, mbytes;;

    /* ļ255uinicodeַ */
    /* Strip duplicated separator */
    for (p = *path; *p == '/' || *p == '\\'; p++) ;
    si = di = 0;
    for (;;) {
        w = p[si++];
        if (w < ' ' || w == '/' || w == '\\')
            break;
        if (di >= 255)
            return RF_ENAMETOOLONG;

#if RF_OPT_HAVE_CODE_PAGE
        /* ִ֧ҳ תΪunicode */
        w &= 0xFF;
        if (isdbcs1((rf_char_t)w)) {
            b = (rf_u8_t)p[si++];
            if (!isdbcs2((rf_char_t)b))
                return RF_ENAMEENCODING;
            w = (w << 8) + b;
        }
        w = a2w(w);
        if (!w)
            return RF_ENAMEENCODING;
#endif
        if (w < 0x80 && rf_chr_chk("\"*:<>\?|\x7F", (rf_char_t)w))
            return RF_EINVAL;

#if RF_OPT_HAVE_CODE_PAGE == 0
        if (w >  0x80)
            return RF_ENAMEENCODING;
#endif

        lfn[di++] = w;
    }
    *path = &p[si];
    cf = (w < ' ') ? NS_LAST : 0;
#if RF_OPT_FS_RPATH
    if ((di == 1 && lfn[di - 1] == '.') || (di == 2 && lfn[di - 1] == '.' && lfn[di - 2] == '.')) {
        lfn[di] = 0;
        for (i = 0; i < 11; i++)
            sfn[i] = (i < di) ? '.' : ' ';
        sfn[i] = cf | NS_DOT;
        return 0;
    }
#endif

    /* Strip trailing spaces and dots */
    while (di) {
        w = lfn[di - 1];
        if (w != ' ' && w != '.')
            break;
        di--;
    }
    if (!di) {
        sfn[0] = '\0';
        return 0;
    }
    lfn[di] = 0;

    /* Create SFN in directory form */
    memset(sfn, ' ', 11);
    /* Strip leading spaces and dots */
    for (si = 0; lfn[si] == ' ' || lfn[si] == '.'; si++) ;
    if (si)
        cf |= NS_LOSS | NS_LFN;
    /* Find extension (di<=si: no extension) */
    while (di && lfn[di - 1] != '.')
        di--;

    mbytes = mb_bytes();
    b = i = 0;
    ni = 8;
    for (;;) {
        w = lfn[si++];
        if (!w)
            break;
        if (w == ' ' || (w == '.' && si != di)) {
            cf |= NS_LOSS | NS_LFN;
            continue;
        }

        if (i >= ni || si == di) {      /* Extension or end of SFN */
            if (ni == 11) {             /* Long extension */
                cf |= NS_LOSS | NS_LFN;
                break;
            }
            if (si != di)
                cf |= NS_LOSS | NS_LFN;   /* Out of 8.3 format */
            if (si > di)
                break;         /* No extension */
            si = di;
            i = 8;
            ni = 11;    /* Enter extension section */
            b <<= 2;
            continue;
        }

        if (w >= 0x80) {
#if RF_OPT_HAVE_CODE_PAGE
            w = w2A(w);
#else
            return RF_ENAMEENCODING;
#endif
        }

        /* Double byte char (always false on SBCS cfg) */
        if (mbytes == 2 && w >= 0x100) {
            if (i >= ni - 1) {
                cf |= NS_LOSS | NS_LFN;
                i = ni;
                continue;
            }
            sfn[i++] = (rf_u8_t)(w >> 8);
        } else {
            if (!w || rf_chr_chk("+,;=[]", (rf_char_t)w)) {
                w = '_';
                cf |= NS_LOSS | NS_LFN;
            } else {
                if (IsUpper(w)) {
                    b |= 2;
                } else {
                    if (IsLower(w)) {
                        b |= 1;
                        w -= 0x20;
                    }
                }
            }
        }
        sfn[i++] = (rf_u8_t)w;
    }

    if ((rf_u8_t)sfn[0] == DDE)
        sfn[0] = NDDE;
    if (ni == 8)
        b <<= 2;
    if ((b & 0x0C) == 0x0C || (b & 0x03) == 0x03)
        cf |= NS_LFN;
    if (!(cf & NS_LFN)) {
        if ((b & 0x03) == 0x01)
            cf |= NS_EXT;
        if ((b & 0x0C) == 0x04)
            cf |= NS_BODY;
    }
    sfn[NS] = cf;
    return 0;

#else   /* Non-LFN configuration */
    rf_u8_t b, c;
    rf_int_t ni, si, i;
    const rf_char_t* p;

    /* ȥ */
    lfn = NULL;
    for (p = *path; *p == '/' || *p == '\\'; p++) ;
    memset(sfn, ' ', 11);
    si = i = b = 0;
    ni = 8;
#if RF_OPT_FS_RPATH
    if (p[si] == '.') {
        for (;;) {
            c = (rf_u8_t)p[si++];
            if (c != '.' || si >= 3)
                break;
            sfn[i++] = c;
        }
        if (c != '/' && c != '\\' && c > ' ')
            return RF_EINVAL;
        *path = &p[si];
        sfn[NS] = (c <= ' ') ? NS_LAST | NS_DOT : NS_DOT;
        return 0;
    }
#endif
    for (;;) {
        c = (rf_u8_t)p[si++];
        if (c <= ' ' || c == '/' || c == '\\')
            break;
        if (c == '.' || i >= ni) {
            if (ni != 8 || c != '.')
                return RF_ENAMETOOLONG;
            i = 8;
            ni = 11;
            b <<= 2;
            continue;
        }
        if (c >= 0x80) {
            b |= 3;
#if RF_OPT_HAVE_CODE_PAGE
            c = e2A(c);
#else
            return RF_ENAMEENCODING;
#endif
        }
#if RF_OPT_HAVE_CODE_PAGE
        if (isdbcs1(c)) {
            rf_u8_t d = p[si++];
            if (!isdbcs2(d) || i >= ni - 1)
                return RF_EINVAL;
            sfn[i++] = c;
            sfn[i++] = d;
            continue;
        }
#endif
        if (rf_chr_chk("\"*+,:;<=>\?[]|\x7F", c))
            return RF_EINVAL;
        if (IsUpper(c)) {
            b |= 2;
        } else {
            if (IsLower(c)) {
                b |= 1;
                c -= 0x20;
            }
        }
        sfn[i++] = c;
    }
    *path = &p[si];
    c = (c <= ' ') ? NS_LAST : 0;

    if (!i)
        return RF_EINVAL;
    if (sfn[0] == DDE)
        sfn[0] = NDDE;
    if (ni == 8)
        b <<= 2;
    /* ļҪ ڴСĸ */
    if ((b & 0x0C) == 0x0C || (b & 0x03) == 0x03)
        return RF_EINVAL;
    if ((b & 0x03) == 0x01)
        c |= NS_EXT;
    if ((b & 0x0C) == 0x04)
        c |= NS_BODY;
    sfn[NS] = c;
    return 0;
#endif
}

#if RF_OPT_FAT_LONG_NAME
rf_void_t fat_gen_numname(rf_char_t* dst, const rf_char_t* src, const rf_wchar_t* lfn, rf_u32_t seq)
{
    rf_char_t ns[8], c;
    rf_u32_t i, j;

    memcpy(dst, src, 11);
    if (seq > 5) {  /* On many collisions, generate a hash number instead of sequential number */
        rf_wchar_t wc;
        rf_u32_t sr = seq;

        while (*lfn) {  /* Create a CRC */
            wc = *lfn++;
            for (i = 0; i < 16; i++) {
                sr = (sr << 1) + (wc & 1);
                wc >>= 1;
                if (sr & 0x10000)
                    sr ^= 0x11021;
            }
        }
        seq = (rf_u32_t)sr;
    }

    /* itoa (hexdecimal) */
    i = 7;
    do {
        c = (seq % 16) + '0';
        if (c > '9')
            c += 7;
        ns[i--] = c;
        seq /= 16;
    } while (seq);
    ns[i] = '~';

    /* Append the number */
    for (j = 0; j < i && dst[j] != ' '; j++) {
#if RF_OPT_HAVE_CODE_PAGE
        if (isdbcs1(dst[j])) {
            if (j == i - 1)
                break;
            j++;
        }
#endif
    }
    do {
        dst[j++] = (i < 8) ? ns[i++] : ' ';
    } while (j < 8);
}
#endif

/* 1   TXT תΪ 1.TXT */
rf_void_t fat_convert_raw_name(rf_char_t* raw_name, rf_int_t nt, rf_char_t* file_name)
{
    rf_int_t i;
    rf_tchar_t c;

    for (i = 0; i < 8; i++) {
        c = raw_name[i];
        if (c == ' ')
            break;
        if (c == NDDE)
            c = (rf_tchar_t)DDE;
        if ((nt & NS_BODY) && IsUpper(c))
            c += 0x20;
        *file_name++ = c;
    }
    if (raw_name[8] != ' ') {
        *file_name++ = '.';
        for (i = 8; i < 11; i++) {
            c = raw_name[i];
            if (c == ' ')
                break;
            if ((nt & NS_EXT) && IsUpper(c))
                c += 0x20;
            *file_name++ = c;
        }
    }
    *file_name = 0;
}

/* ѶļΪ fatŵĸʽ */
#if RF_OPT_USE_SET_NAME
rf_int_t fat_convert_short_name(const rf_char_t* name, rf_char_t* sfn)
{
    rf_u8_t b, c;
    rf_int_t ni, si, i;
    const rf_char_t* p = name;

    memset(sfn, ' ', 11);
    si = i = b = 0;
    ni = 8;
    for (;;) {
        c = (rf_u8_t)p[si++];
        if (c <= ' ' || c == '/' || c == '\\')
            break;   /* Break on end of segment */
        if (c == '.' || i >= ni) {
            if (ni != 8 || c != '.')
                return RF_EINVAL;
            i = 8;
            ni = 11;
            b <<= 2;
            continue;
        }
        if (c >= 0x80) {                /* Extended char? */
            b |= 3;                     /* Eliminate NT flag */
#if RF_OPT_HAVE_CODE_PAGE
            c = (rf_u8_t)e2A(c);                 /* To upper extended chars (SBCS cfg) */
#else
            return RF_ENAMEENCODING;
#endif
        }

        if (isdbcs1(c)) {               /* Check if it is a DBC 1st byte (always false on SBCS cfg) */
#if RF_OPT_HAVE_CODE_PAGE
            rf_u8_t d = (rf_u8_t)p[si++];
            if (!isdbcs2(d) || i >= ni - 1)
                return RF_EINVAL;
            sfn[i++] = c;
            sfn[i++] = d;
#endif
        } else {                        /* Single byte code */
            if (rf_chr_chk("\"*+,:;<=>\?[]|\x7F", c))
                return RF_EINVAL;
            if (IsUpper(c)) {           /* ASCII large capital? */
                b |= 2;
            } else {
                if (IsLower(c)) {       /* ASCII small capital? */
                    b |= 1;
                    c -= 0x20;
                }
            }
            sfn[i++] = c;
        }
    }
    c = (c <= ' ') ? NS_LAST : 0;       /* Set last segment flag if end of path */

    if (!i)
        return RF_EINVAL;           /* Reject nul string */
    if ((rf_u8_t)sfn[0] == DDE)
        sfn[0] = NDDE;   /* When first char collides with DDE, replace it with 0x05 */

    if (ni == 8)
        b <<= 2;
    /* ļڴС ЧĶļ */
    if ((b & 0x0C) == 0x0C || (b & 0x03) == 0x03)
        return RF_EINVAL;
    if ((b & 0x03) == 0x01)
        c |= NS_EXT;    /* NT flag (Name extension has only small capital) */
    if ((b & 0x0C) == 0x04)
        c |= NS_BODY;   /* NT flag (Name body has only small capital) */

    sfn[NS] = c;                            /* Store NT flag, File name is created */
    return 0;
}
#endif

#if RF_OPT_USE_NTFS || RF_OPT_USE_EXT2
rf_int_t path_get_fn(const rf_char_t** path, rf_char_t* name, rf_int_t len, rf_int_t cp)
{
    const rf_char_t* p;
    rf_int_t tmp, si, di;
    rf_u16_t w;

    for (p = *path; *p == '/' || *p == '\\'; p++);
    if (p[0] == '\0') {
        if (cp == RF_CODE_PAGE_UCS2L || cp == RF_CODE_PAGE_UCS2B)
            *(rf_wchar_t*)name = (rf_wchar_t)'\0';
        else *name = '\0';
        return RF_ESUCCESS;
    }

    si = di = 0;
    for (;;) {
        w = (rf_u8_t)p[si++];
        if (w < ' ' || w == '/' || w == '\\') {
            if (w == '\0')
                si--;
            break;
        }
        RF_PERROR("name too len\n", di >= len, return RF_EINVAL);

#if RF_OPT_HAVE_CODE_PAGE
        if (cp == RF_CODE_PAGE_UCS2L || cp == RF_CODE_PAGE_UCS2B || cp == RF_CODE_PAGE_UTF8) {
            w &= 0xFF;
            if (isdbcs1((rf_u8_t)w)) {
                rf_u8_t b;
                b = (rf_u8_t)p[si++];
                if (!isdbcs2((rf_char_t)b))
                    return RF_ENAMEENCODING;
                w = (w << 8) + b;
            }
            w = a2w(w);
            if (!w)
                return RF_ENAMEENCODING;

            if (w < 0x80 && rf_chr_chk("\"*:<>\?|\x7F", (rf_char_t)w))
                return RF_EINVAL;
            if (cp == RF_CODE_PAGE_UTF8) {
                wchar_to_utf8(w, name + di, len - di, &tmp);
                if (tmp == 0)
                    return RF_ENAMEENCODING;
                di += tmp;
            } else {
                *(rf_wchar_t*)(name + di) = w;
                di += 2;
            }
            continue;
        }
#else
        if (w >  0x80)
            return RF_ENAMEENCODING;
        if (cp == RF_CODE_PAGE_UCS2L || cp == RF_CODE_PAGE_UCS2B) {
            *(rf_wchar_t*)(name + di) = w;
            di += 2;
        } else {
            *(name + di++) = w;
        }
#endif
    }

    *path = &p[si];
#if RF_OPT_FS_RPATH
    if (cp == RF_CODE_PAGE_UCS2L || cp == RF_CODE_PAGE_UCS2B) {
        if ((di == 2 && *((rf_wchar_t*)name) == '.') ||
                ((di == 4 && *((rf_wchar_t*)name) == '.' &&  *((rf_wchar_t*)name + 1) == '.'))) {
            *((rf_wchar_t*)name + (di / 2)) = 0;
            return 0;
        }
    } else {

        if ((di == 1 && *name == '.') || ((di == 2 && *name == '.' &&  *(name + 1) == '.'))) {
            *(name + di) = 0;
            return 0;
        }
    }
#endif
    /* ɾβո͵ */
    if (cp == RF_CODE_PAGE_UCS2L || cp == RF_CODE_PAGE_UCS2B) {
        di >>= 1;
        while (di) {
            w = ((rf_wchar_t*)name)[di - 1];
            if (w != ' ' && w != '.')
                break;
            di--;
        }
        if (!di)
            return RF_EINVAL;
        ((rf_wchar_t*)name)[di] = 0;
    } else {
        while (di) {
            w = name[di - 1];
            if (w != ' ' && w != '.')
                break;
            di--;
        }
        if (!di)
            return RF_EINVAL;
        name[di] = 0;
    }
    return RF_ESUCCESS;
}
#endif
