/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

#if RF_OPT_USE_NTFS
rf_wchar_t rf_wchr_chk(const rf_wchar_t* str, rf_wchar_t chr)
{
    while (*str && *str != chr)
        str++;
    return *str;
}

rf_wchar_t* rf_a2w_strcpy(rf_wchar_t* dst, const rf_char_t* src)
{
    rf_wchar_t* cp = dst;
    while (*cp++ = (rf_wchar_t)(*src++))
        ;               /* Copy src over dst */
    return (dst);
}

rf_bool_t rf_wstr_contain_wild_cards(const rf_wchar_t* string)
{
    while (*string) {
        if (*string == '*' || *string == '?')
            return TRUE;
        string++;
    }
    return FALSE;
}

rf_int_t rf_wstrncmp_buf(rf_u8_t* buf, rf_u16_t idx, rf_wchar_t* b, rf_u16_t n)
{
    rf_u32_t i;
    rf_wchar_t c;

    for (i = 0; i < n; i++) {
        c = u16_read(buf, (rf_u16_t)(idx + (i << 1)));
        if (c < * (b + i))
            return -1;
        if (*(b + i) < c)
            return 1;
        if (*(b + i) == 0)
            break;
    }
    return 0;
}

rf_wchar_t* rf_wstrcpy(rf_wchar_t* dst, const rf_wchar_t* src)
{
    rf_wchar_t* cp = dst;
    while (*cp++ = *src++)
        ;               /* Copy src over dst */
    return (dst);
}

#endif

#if RF_OPT_USE_NTFS || (RF_OPT_USE_FAT && (RF_OPT_FAT_LONG_NAME || RF_OPT_USE_GET_SET_LONG_NAME))
rf_size_t rf_wstrlen(const rf_wchar_t* str)
{
    const rf_wchar_t* s = (const rf_wchar_t*)str;
    if (str == NULL)
        return 0;

    while (*s)
        s++;
    return (rf_size_t)(s - str);
}
#endif

#if RF_OPT_USE_FAT && RF_OPT_FAT_LONG_NAME
rf_int_t rf_wcscmp(const rf_wchar_t* src, const rf_wchar_t* dst)
{
    rf_int_t ret = 0 ;

    while (!(ret = *(rf_wchar_t*)src - * (rf_wchar_t*)dst) && *dst)
        ++src, ++dst;

    if (ret < 0)
        ret = -1;
    else if (ret > 0)
        ret = 1;
    return (ret);
}
#endif

rf_bool_t rf_str_contain_wild_cards(const rf_char_t* string)
{
    while (*string) {
        if (*string == '*' || *string == '?')
            return TRUE;
        string++;
    }
    return FALSE;
}

rf_char_t rf_chr_chk(const rf_char_t* str, rf_char_t chr)
{
    while (*str && *str != chr)
        str++;
    return *str;
}

#if RF_OPT_USE_GETCWD
rf_int_t rf_strcpy_s(rf_char_t* dest, rf_int_t len, const rf_char_t* src)
{
    if (len >= strlen(src) + 1) {
        strcpy(dest, src);
        return 0;
    }
    return RF_EBUFSMALL;
}

rf_int_t rf_strcat_s(rf_char_t* dest, rf_int_t len, const rf_char_t* src)
{
    rf_size_t dlen;

    dlen = strlen(dest);
    if (len > dlen + strlen(src) + 1) {
        strcpy(dest + dlen, src);
        return 0;
    }
    return RF_EBUFSMALL;
}

rf_int_t rf_str_replace_all(rf_char_t* str, rf_char_t old_char, rf_char_t new_char)
{
    rf_size_t i, len;

    len = strlen(str);
    for (i = 0; i < len; i++) {
        if (str[i] == old_char)
            str[i] = new_char;
    }
    return 0;
}
#endif
