/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

#ifdef RF_DEBUG
rf_bool_t check_order_word(rf_void_t)
{
    rf_u16_t x = 0x1234;
    rf_u8_t const* p = (rf_u8_t const*)&x;

#if RF_OPT_CPU_BIG_ENDIAN
    return (p[0] == 0x12 && p[1] == 0x34) ? TRUE : FALSE;
#else
    return (p[0] == 0x34 && p[1] == 0x12) ? TRUE : FALSE;
#endif
}
#endif

/**
 * ffs - find first bit set
 * @x: the word to search
 *
 * This is defined the same way as
 * the libc and compiler builtin ffs routines, therefore
 * differs in spirit from the above ffz (man ffs).
 */
rf_u8_t rf_ffs(rf_u32_t x)
{
    rf_u8_t pos;
    pos = 0;
    while (x != 0) {
        if (!(x & 0x1))
            pos++;
        x = x >> 1;
    }
    return pos;
}

#if (RF_OPT_USE_FAT && (RF_OPT_FAT_LONG_NAME || RF_OPT_USE_GET_SET_LONG_NAME))
rf_u8_t fat_long_file_name_sum(const rf_u8_t* p)
{
    rf_u8_t FcbNameLen;
    rf_u8_t Sum;

    Sum = 0;
    for (FcbNameLen = 0; FcbNameLen != 11; FcbNameLen++) {
        Sum = ((Sum & 1) ? 0x80 : 0) + (Sum >> 1) + *p++;
    }
    return (Sum);
}
#endif

