/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_EXT_H__
#define __RF_EXT_H__

#define EXT2_NDIR_BLOCKS            12

/* The good old (original) format */
#define EXT2_GOOD_OLD_REV           0
/* V2 format w/ dynamic inode sizes */
#define EXT2_DYNAMIC_REV            1

#define EXT2_CURRENT_REV            EXT2_GOOD_OLD_REV
#define EXT2_MAX_SUPP_REV           EXT2_DYNAMIC_REV

#define EXT2_GOOD_OLD_INODE_SIZE    128

/* Block Size Management */
#define EXT2_MIN_BLOCK_SIZE         1024
#define EXT2_MAX_BLOCK_SIZE         4096
#define EXT2_MIN_BLOCK_LOG_SIZE     10

/*Inode flags  */
#define EXT2_SECRM_FL               0x00000001 /* Secure deletion */
#define EXT2_UNRM_FL                0x00000002 /* Undelete */
#define EXT2_COMPR_FL               0x00000004 /* Compress file */
#define EXT2_SYNC_FL                0x00000008 /* Synchronous updates */
#define EXT2_IMMUTABLE_FL           0x00000010 /* Immutable file */
#define EXT2_APPEND_FL              0x00000020 /* writes to file may only append */
#define EXT2_NODUMP_FL              0x00000040 /* do not dump file */
#define EXT2_NOATIME_FL             0x00000080 /* do not update atime */

/* Reserved for compression usage... */
#define EXT2_DIRTY_FL               0x00000100
#define EXT2_COMPRBLK_FL            0x00000200 /* One or more compressed clusters */
#define EXT2_NOCOMP_FL              0x00000400 /* Don't compress */
#define EXT2_ECOMPR_FL              0x00000800 /* Compression error */

/* End compression flags --- maybe not all used */
#define EXT2_BTREE_FL               0x00001000 /* btree format dir */
#define EXT2_IMAGIC_FL              0x00002000 /* AFS directory */
#define EXT2_JOURNAL_DATA_FL        0x00004000 /* file data should be journaled */
#define EXT2_NOTAIL_FL              0x00008000 /* file tail should not be merged */
#define EXT2_DIRSYNC_FL             0x00010000 /* dirsync behaviour (directories only) */
#define EXT2_TOPDIR_FL              0x00020000 /* Top of directory hierarchies*/
#define EXT2_HUGE_FILE_FL           0x00040000 /* Set to each huge file */
#define EXT2_EXTENTS_FL             0x00080000 /* Inode uses extents */
#define EXT2_RESERVED_FL            0x80000000 /* reserved for ext2 lib */

#define S_IFMT   0x0F000            /* 017 0000 */
#define S_IFSOCK 0x0C000            /* 014 0000 */
#define S_IFLNK  0x0A000            /* 012 0000 */
#define S_IFREG  0x08000            /* 010 0000 */
#define S_IFBLK  0x06000            /* 006 0000 */
#define S_IFDIR  0x04000            /* 004 0000 */
#define S_IFCHR  0x02000            /* 002 0000 */
#define S_IFIFO  0x01000            /* 001 0000 */
#define S_ISUID  0x00800            /* 000 4000 */
#define S_ISGID  0x00400            /* 000 2000 */
#define S_ISVTX  0x00200            /* 000 1000 */

#define S_ISREG(m)      (((m) & S_IFMT) == S_IFREG)
#define S_ISSOCK(m)     (((m) & S_IFMT) == S_IFSOCK)
#define S_ISLNK(m)      (((m) & S_IFMT) == S_IFLNK)
#define S_ISFIL(m)      (((m) & S_IFMT) == S_IFFIL)
#define S_ISBLK(m)      (((m) & S_IFMT) == S_IFBLK)
#define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
#define S_ISCHR(m)      (((m) & S_IFMT) == S_IFCHR)
#define S_ISFIFO(m)     (((m) & S_IFMT) == S_IFIFO)

#define S_IPERMISSION_MASK          0x1FF

#define S_IRWXU  0x1C0              /* 0 0700 */
#define S_IRWNU  0x180              /* 0 0600 */
#define S_IRUSR  0x100              /* 0 0400 */
#define S_IWUSR  0x080              /* 0 0200 */
#define S_IXUSR  0x040              /* 0 0100 */

#define S_IRWXG  0x038              /* 0 0070 */
#define S_IRWNG  0x030              /* 0 0060 */
#define S_IRGRP  0x020              /* 0 0040 */
#define S_IWGRP  0x010              /* 0 0020 */
#define S_IXGRP  0x008              /* 0 0010 */

#define S_IRWXO  0x007              /* 0 0007 */
#define S_IRWNO  0x006              /* 0 0006 */
#define S_IROTH  0x004              /* 0 0004 */
#define S_IWOTH  0x002              /* 0 0002 */
#define S_IXOTH  0x001              /* 0 0001 */

#define S_IRWXUGO   (S_IRWXU|S_IRWXG|S_IRWXO)
#define S_IALLUGO   (S_ISUID|S_ISGID|S_ISVTX|S_IRWXUGO)
#define S_IRUGO     (S_IRUSR|S_IRGRP|S_IROTH)
#define S_IWUGO     (S_IWUSR|S_IWGRP|S_IWOTH)
#define S_IXUGO     (S_IXUSR|S_IXGRP|S_IXOTH)
#define S_IFATTR    (S_IRWNU|S_IRWNG|S_IRWNO)

#define S_ISREADABLE(m)    (((m) & S_IPERMISSION_MASK) == (S_IRUSR | S_IRGRP | S_IROTH))
#define S_ISWRITABLE(m)    (((m) & S_IPERMISSION_MASK) == (S_IWUSR | S_IWGRP | S_IWOTH))

#define Ext2SetReadable(m) (m) = ((m) | (S_IRUSR | S_IRGRP | S_IROTH))
#define Ext2SetWritable(m) (m) = ((m) | (S_IWUSR | S_IWGRP | S_IWOTH))
#define Ext2SetOwnerWritable(m) (m) = ((m) | (S_IWUSR))

#define Ext2SetReadOnly(m) (m) = ((m) & (~(S_IWUSR | S_IWGRP | S_IWOTH)))
#define Ext2IsReadOnly(m)  (!((m) & (S_IWUSR | S_IWGRP | S_IWOTH)))

#endif
