/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __EXT2_VOLUME_H__
#define __EXT2_VOLUME_H__

#define EXT2_FEATURE_COMPAT_DIR_PREALLOC    0x0001
#define EXT2_FEATURE_COMPAT_IMAGIC_INODES   0x0002
#define EXT3_FEATURE_COMPAT_HAS_JOURNAL     0x0004
#define EXT2_FEATURE_COMPAT_EXT_ATTR        0x0008
#define EXT2_FEATURE_COMPAT_RESIZE_INO      0x0010
#define EXT2_FEATURE_COMPAT_DIR_INDEX       0x0020

#define EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER 0x0001
#define EXT2_FEATURE_RO_COMPAT_LARGE_FILE   0x0002
#define EXT2_FEATURE_RO_COMPAT_BTREE_DIR    0x0004
#define EXT4_FEATURE_RO_COMPAT_GDT_CSUM     0x0010
#define EXT4_FEATURE_RO_COMPAT_DIR_NLINK    0x0020
#define EXT4_FEATURE_RO_COMPAT_EXTRA_ISIZE  0x0040

#define EXT2_FEATURE_INCOMPAT_COMPRESSION   0x0001
#define EXT2_FEATURE_INCOMPAT_FILETYPE      0x0002
#define EXT3_FEATURE_INCOMPAT_RECOVER       0x0004
#define EXT3_FEATURE_INCOMPAT_JOURNAL_DEV   0x0008
#define EXT3_FEATURE_INCOMPAT_META_BG       0x0010

struct _rf_ext2_volume_t {
    /* ļϵͳ */
    rf_u8_t fs_type: 4;
    /* 򿪱־ */
    rf_u8_t open_flag: 1;
    /*  ģʽ 0 1д */
    rf_u8_t access_flag: 1;
#if RF_OPT_FS_READONLY == 0
    /* µǰ־λ */
    rf_u8_t write_flag: 1;
#endif
#if RF_OPT_USE_MALLOC == 1
    list_t list;
#endif
#if RF_OPT_USE_VOLUME_CTX == 1
    rf_void_t* ctx;
#endif
    /* 豸ָ */
    rf_handle_t dev_handle;
#if RF_OPT_USE_MALLOC == 1
    /*  */
    rf_u8_t* sector_buffer;
#else
    rf_u8_t sector_buffer[RF_OPT_MAX_SECTOR_SIZE];
#endif

    /* ǰҳ */
    rf_int_t code_page;
    /* ǰĿ¼ ڵ */
    rf_fentry_t current_dir_entry;
    /* Ŀ¼ ڵ */
    rf_fentry_t root_dir_entry;
    /* С */
    rf_u16_t sector_size;
    /* ֽڵλ λ */
    rf_u8_t sector_size_bits;
    /* һλ λ */
    rf_u8_t block_size_bits;
    /* һؼλ λ */
    rf_u8_t sectors_per_cluster_bits;

    /* һ  */
    rf_u8_t     sectors_per_cluster;
    /* ÿֽ */
    rf_u32_t    block_size;
    /* ܹжٸ */
    rf_u32_t    blocks_count;
    /* п */
    rf_u32_t    free_blocks_count;
    /* ÿӵеĽڵ */
    rf_u32_t    inodes_per_group;
    /* ÿڵֽ */
    rf_u32_t    inode_size;
    /* ܹжٸ */
    rf_u32_t    total_groups;
    /* һʼ */
    rf_u32_t    group_start_sector;
};
typedef struct _rf_ext2_volume_t rf_ext2_volume_t, *rf_ext2_volume_ref_t;

rf_int_t ext2_volume_open(rf_ext2_volume_ref_t vol, rf_mount_param_t* param);
rf_int_t ext2_volume_close(rf_ext2_volume_ref_t vol);
rf_int_t ext2_volume_stat(rf_ext2_volume_ref_t vol, rf_statfs_ref_t pvs);
rf_int_t ext2_volume_get_label(rf_ext2_volume_ref_t vol, const rf_char_t* buf, rf_int_t len);
rf_int_t ext2_volume_set_label(rf_ext2_volume_ref_t vol, const rf_char_t* buf);
rf_int_t ext2_volume_set_current_dir(rf_ext2_volume_ref_t vol, rf_fentry_t* entry);

#endif
