/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __FAT_FILE_H__
#define __FAT_FILE_H__

struct _rf_fat_file_t {
    /* 򿪻߹ر ־λ */
    rf_u8_t             open_flag: 1;
    /* 0ֻ 1д 2дȨ */
    rf_u8_t             access_flag: 2;
    /* 1 ļ־  */
    rf_u8_t             sector_buf_flag: 1;
    /* 1 ǰΪĿ¼ */
    rf_u8_t             root_dir_flag: 1;
    /* Ŀ¼ڷ */
    rf_fat_volume_ref_t vol;
#if RF_OPT_USE_FILE_CTX
    /* ļ */
    rf_void_ref_t       ctx;
#endif
    /* ̬malloc fileʱ ʹ */
#if RF_OPT_USE_MALLOC
    list_t              list;
#endif

    /* һѼ ־λ */
    rf_u8_t             first_gather_cluester_bit: 1;
    /* дǷѼ ־λ */
    rf_u8_t             gather_all_clust_bit: 1;
#if RF_OPT_FS_READONLY == 0
    /* ļȸı־λ */
    rf_u8_t             attrib_change_bit: 1;
    /* ļĿ¼ Ҫ´ر־λ */
    rf_u8_t             dir_end_clst_bit: 1;
#endif

    /* ļ */
    rf_u8_t             attribute;
    /* ļʼ fat12 16 ¸ı*/
    rf_u32_t            start_cluster;
    /* ǰļ ϼĿ¼ ʼ*/
    rf_u32_t            updir_start_cluster;
    /* ļϼĿ¼ƫ */
    rf_u32_t            up_dir_offset;
    /* ļ */ /* fat12 16 Ŀ¼Ϊ ȡƫ */
    rf_u32_t            size;
    /* ļĿ¼ǰƫ */
    rf_u32_t            current_offset;

    /* 32λΪ 32λΪڴƫ */
    rf_fentry_t         entry;
    /* ļؽṹ */
    rf_chain_idx_t      clust;

#if RF_OPT_USE_EXFAT
    rf_u8_t             stat;
    rf_size_t           objsize;        /* Object size (valid when sclust != 0) */
    rf_u32_t            n_cont;         /* Size of first fragment - 1 (valid when stat == 3) */
    rf_u32_t            n_frag;         /* Size of last fragment needs to be written to FAT (valid when not zero) */
    rf_u32_t            c_scl;          /* Containing directory start cluster (valid when sclust != 0) */
    rf_u32_t            c_size;         /* b31-b8:Size of containing directory, b7-b0: Chain status (valid when c_scl != 0) */
    rf_u32_t            c_ofs;          /* Offset in the containing directory (valid when file object and sclust != 0) */
#endif
};
typedef struct _rf_fat_file_t rf_fat_file_t, *rf_fat_file_ref_t;

rf_bool_t fat_check_file_access(rf_fat_file_ref_t file);
rf_bool_t check_start_cluster_access(rf_fat_volume_ref_t vol, rf_u32_t clst, rf_int_t flag);

rf_int_t fat_file_open(rf_fat_file_ref_t file, const rf_char_t* name, rf_int_t flags);
rf_int_t fat_file_raw_open(rf_fat_file_ref_t file, rf_fentry_t* fentry, rf_int_t flag);
rf_int_t fat_file_close(rf_fat_file_ref_t pf);
rf_int_t fat_file_lseek(rf_fat_file_ref_t file, rf_u32_t offset, rf_int_t origin);
rf_int_t fat_file_get_name(rf_fat_file_ref_t file, rf_char_t* buf, rf_u32_t len);
rf_int_t fat_file_get_long_name(rf_fat_file_ref_t file, rf_wchar_t* buf, rf_u32_t len);
rf_int_t fat_file_get_stat(rf_fat_file_ref_t file, rf_stat_ref_t st);
rf_int_t fat_file_get_ftime(rf_fat_file_ref_t file, rf_file_ftime_ref_t ft);
rf_int_t fat_file_get_ltime(rf_fat_file_ref_t file, rf_file_ltime_ref_t lt);
rf_int_t fat_file_read_dir(rf_fat_file_ref_t dir, const rf_char_t* pattern, rf_dirent_ref_t pfd, rf_int_t flag);
rf_int_t fat_file_read(rf_fat_file_ref_t file, rf_void_t* buf, rf_u32_t len, rf_u32_t* plen);
rf_int_t fat_file_find_sfn(rf_fat_file_ref_t file, const rf_char_t* sfn, rf_u32_t open_flag);

#if RF_OPT_FS_READONLY == 0
rf_u32_t fat_time_current(rf_void_t);
rf_int_t fat_file_set_stat(rf_fat_file_ref_t file, rf_file_ftime_ref_t st);
rf_int_t fat_file_set_attribute(rf_fat_file_ref_t file, rf_u32_t attribute);
rf_int_t fat_file_set_ftime(rf_fat_file_ref_t file, rf_file_ftime_ref_t st);
rf_int_t fat_file_set_ltime(rf_fat_file_ref_t file, rf_file_ltime_ref_t lt);
rf_int_t fat_file_set_name(rf_fat_file_ref_t file, const rf_char_t* buf);
rf_int_t fat_file_set_long_name(rf_fat_file_ref_t file, const rf_wchar_t* buf);
rf_int_t fat_file_rm_long_name(rf_fat_file_ref_t file);
rf_int_t fat_file_sync(rf_fat_file_ref_t file);
rf_int_t fat_file_write(rf_fat_file_ref_t file, rf_void_t* buf, rf_u32_t len, rf_u32_t* plen);
rf_int_t fat_file_truncate(rf_fat_file_ref_t file, rf_u32_t length);
rf_int_t fat_file_delete(rf_fat_file_ref_t file);
rf_int_t fat_file_rename(rf_fat_file_ref_t oldfile, rf_fat_file_ref_t newfile, const rf_char_t* oldname, const rf_char_t* newname);
#endif

#endif
