/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_DRIVER_H__
#define __RF_DRIVER_H__

#define RF_CMD_GET_SECTOR_COUNT     1     /* ȡС */
#define RF_CMD_GET_SECTOR_SIZE      2     /* ȡ豸ֽڴС */
#define RF_CMD_GET_BLOCK_SIZE       3     /* ȡС ʽʹ */
#define RF_CMD_CTRL_ERASE_SECTOR    4     /* 豸 ʽʹ */
#define RF_CMD_CTRL_SYNC            5     /* ˢݵ */

#if RF_OPT_DYNAMIC_LIB
typedef rf_int_t (*rf_drv_open_t)(const rf_char_t* device_name, rf_int_t flags, rf_void_t* ctx, rf_handle_t* hdev);
typedef rf_int_t (*rf_drv_close_t)(rf_handle_t dev);
typedef rf_int_t (*rf_drv_ioctl_t)(rf_handle_t dev, rf_int_t cmd, rf_void_t* buf, rf_int_t len) reentrant;
typedef rf_int_t (*rf_drv_read_t)(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf) reentrant;
typedef rf_int_t (*rf_drv_write_t)(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf) reentrant;
struct _rf_drv_t {
    rf_drv_open_t open;
    rf_drv_close_t close;
    rf_drv_ioctl_t ioctl;
    rf_drv_read_t read;
#if RF_OPT_FS_READONLY == 0
    rf_drv_write_t write;
#endif
};
typedef struct _rf_drv_t rf_drv_t, *rf_drv_ref_t;
extern rf_drv_open_t rf_disk_open;
extern rf_drv_close_t rf_disk_close;
extern rf_drv_ioctl_t rf_disk_ioctl;
extern rf_drv_read_t rf_disk_read;
#if RF_OPT_FS_READONLY == 0
extern rf_drv_write_t rf_disk_write;
#endif
#else
rf_int_t rf_disk_init(void);
rf_int_t rf_disk_open(const rf_char_t* device_name, rf_int_t flags, rf_void_t* ctx, rf_handle_t* hdev);
rf_int_t rf_disk_close(rf_handle_t dev);
rf_int_t rf_disk_ioctl(rf_handle_t dev, rf_int_t cmd, rf_void_t* buf, rf_int_t len);
rf_int_t rf_disk_read(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf);
rf_int_t rf_disk_write(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf);
#endif

rf_int_t rf_getlocaltime(rf_tm_ref_t tm);

#endif
