/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_KEYWORD_H__
#define __RF_KEYWORD_H__

#ifndef __cplusplus
#   define __rf_register__                      register
#else
#   define __rf_register__
#endif
#define __rf_volatile__                         volatile
#define __rf_func__                             __FUNCTION__
#define __rf_file__                             __FILE__
#define __rf_line__                             __LINE__

#if defined(RF_COMPILER_IS_MSVC)

#   define __rf_asm__                           __asm
#   define __rf_inline__                        __inline
#   define __rf_inline_force__                  __forceinline
#   define __rf_cdecl__                         __cdecl
#   define __rf_stdcall__                       __stdcall
#   define __rf_fastcall__                      __fastcall
#   define __rf_thiscall__                      __thiscall
#   define __rf_packed__
#   define __rf_aligned__(a)                    __declspec(align(a))

#elif defined(RF_COMPILER_IS_GCC)

#   define __rf_asm__                           __asm__
#   define __rf_inline__                        __inline__
#   define __rf_inline_force__                  __inline__ __attribute__((always_inline))
#   define __rf_packed__                        __attribute__((packed, aligned(1)))
#   define __rf_aligned__(a)                    __attribute__((aligned(a)))
/* gcc will generate attribute ignored warning */
#   if defined(__x86_64) \
    || defined(__amd64__) \
    || defined(__amd64) \
    || defined(_M_IA64) \
    || defined(_M_X64)
#       define __rf_cdecl__
#       define __rf_stdcall__
#       define __rf_fastcall__
#       define __rf_thiscall__
#   else
#       define __rf_cdecl__                     __attribute__((__cdecl__))
#       define __rf_stdcall__                   __attribute__((__stdcall__))
#       define __rf_fastcall__                  __attribute__((__fastcall__))
#       define __rf_thiscall__                  __attribute__((__thiscall__))
#   endif
#else

#   define __rf_asm__
#   define __rf_inline__                        inline
#   define __rf_inline_force__                  inline
#   define __rf_func__                          __FUNCTION__
#   define __rf_file__                          __FILE__
#   define __rf_line__                          __LINE__

#   define __rf_cdecl__
#   define __rf_stdcall__
#   define __rf_fastcall__
#   define __rf_thiscall__
#   define __rf_packed__
#   define __rf_aligned__(a)

#endif

#if defined(RF_COMPILER_IS_MSVC)
#   define __rf_export__                        __declspec(dllexport)
#elif defined(RF_COMPILER_IS_GCC) && ((__GNUC__ >= 4) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3))
#   define __rf_export__                        __attribute__((visibility("default")))
#else
#   define __rf_export__
#endif

#define RF_API __rf_export__ extern

#endif
