/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_STDIO_H__
#define __RF_STDIO_H__

/*
#define RF_BUFSIZ      512
*/

#define RF_IOREAD         0x0001
#define RF_IOWRT          0x0002
#define RF_IOEOF          0x0010
#define RF_IOERR          0x0020
#define RF_IOSTRG         0x0040
#define RF_IORW           0x0080
#define RF_IOAPPEND       0x2000
#define RF_IOTEXT         0x4000
#define RF_IOBINARY       0x8000

#define RF_IOFBF          0x0000
#define RF_IONBF          0x0004
#define RF_IOMYBUF        0x0008
#define RF_IOLBF          0x0040

#define RF_EOF           -1

#define rf_feof(_stream)     ((_stream)->_flag & RF_IOEOF)
#define rf_ferror(_stream)   ((_stream)->_flag & RF_IOERR)
#define rf_fileno(_stream)  ((_stream)->_file)
#define rf_getc(_stream)     (--(_stream)->_cnt >= 0 \
                ? 0xff & *(_stream)->_ptr++ : rf_filbuf(_stream))
#define rf_putc(_c,_stream)  (--(_stream)->_cnt >= 0 \
                ? 0xff & (*(_stream)->_ptr++ = (char)(_c)) :  rf_flsbuf((_c),(_stream)))
#define rf_getchar()         rf_getc(stdin)
#define rf_putchar(_c)       rf_putc((_c),stdout)

#define rf_getc_lk(_stream)       (--(_stream)->_cnt >= 0 ? 0xff & *(_stream)->_ptr++ : _filbuf(_stream))
#define rf_putc_lk(_c,_stream)    (--(_stream)->_cnt >= 0 ? 0xff & (*(_stream)->_ptr++ = (char)(_c)) :  rf_flsbuf((_c),(_stream)))

/* ------------ļstdio api---------- */
rf_int_t rf_stdio_init(rf_void_t);
rf_int_t rf_stdio_exit(rf_void_t);

RF_API RF_FILE* rf_fopen(const rf_char_t* file, const rf_char_t* mode);
RF_API rf_int_t  rf_fclose(RF_FILE* str);
RF_API rf_int_t rf_fcloseall(rf_void_t);

RF_API RF_FILE* rf_fdopen(rf_int_t filedes, const rf_char_t* mode);
RF_API rf_size_t rf_fread(rf_void_t* buffer, rf_size_t size, rf_size_t num, RF_FILE* stream);

RF_API rf_int_t rf_fputs(const rf_char_t* string, RF_FILE* stream);
RF_API rf_size_t rf_fwrite(const rf_void_t* buffer, rf_size_t size, rf_size_t num, RF_FILE* stream);
RF_API rf_int_t rf_fflush(RF_FILE* str);

RF_API rf_int_t rf_rewind(RF_FILE* str);
RF_API rf_off_t rf_ftell(RF_FILE* str);
RF_API rf_int_t rf_fseek(RF_FILE* str, rf_off_t offset, rf_int_t whence);

#endif
