/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_STRUCT_H__
#define __RF_STRUCT_H__

struct  rf_part {
    rf_lloff_t start;                       /* ʼƫ */
    rf_lloff_t size;                        /* С */
};
typedef struct rf_part rf_part_t, *rf_part_ref_t;

struct  rf_part_param {
    rf_int_t    sector_size;                /* Сֽ 0ΪԶȡȡ */
    rf_int_t    count;                      /* Ч */
    rf_part_t   part[RF_MAX_DISK_PART];     /* ŷṹ */
};
typedef struct rf_part_param rf_part_param_t, *rf_part_param_ref_t;

struct _rf_fentry {
    rf_lloff_t  sector;                     /* ļƫ */
    rf_u16_t    offset;                     /* ļƫ */
};
typedef struct _rf_fentry rf_fentry_t, *rf_fentry_ref_t;

struct _rf_large_uint_t {
    rf_u32_t    low_part;
    rf_u32_t    high_part;
};
typedef struct _rf_large_uint_t rf_large_uint_t, *rf_large_uint_ref_t;

typedef struct _rf_version_t {
    rf_u8_t     major;
    rf_u8_t     minor;
    rf_u8_t     alter;
    rf_u32_t    build;
} rf_version_t;

struct _rf_tm_t {
    rf_u16_t    tm_year;    /*  [1601 - 30827] */
    rf_u16_t    tm_mon;     /* · ȡֵΪ[1,12] */
    rf_u16_t    tm_mday;    /* һе - ȡֵΪ[1,31] */
    rf_u16_t    tm_hour;    /* ʱ - ȡֵΪ[0,23] */
    rf_u16_t    tm_min;     /*  - ȡֵΪ[0,59] */
    rf_u16_t    tm_sec;     /*  CȡֵΪ[0,59] */
};
typedef struct _rf_tm_t rf_tm_t, *rf_tm_ref_t;

struct _rf_ftime_t {
    rf_u32_t    low_date_time;
    rf_u32_t    high_date_time;
};
typedef struct _rf_ftime_t rf_ftime_t, *rf_ftime_ref_t;

struct rf_mount_ctx {
    rf_int_t    sector_size;    /* ָ豸С ֽ */
    rf_lloff_t  volume_offset;  /* ƫ */
#if RF_OPT_HAVE_CODE_PAGE
    rf_char_t*   code_page;     /* fat ext2 ļϵͳĴҳ exfat ntfsҪôҳ */
#endif
};
typedef struct rf_mount_ctx rf_mount_ctx_t, *rf_mount_ctx_ref_t;

struct rf_mount_param {
    const rf_char_t*     device_name;
    rf_int_t            flags;
    rf_mount_ctx_t*      ctx;
};
typedef struct rf_mount_param rf_mount_param_t, *rf_mount_param_ref_t;

/*
rf_fat_mkfs_param cluster_size
FAT16 ֧ 512102420484096819216K32K64K (128K256K ڴ 512 ֽڵ)
FAT32 ֧ 512102420484096819216K32K64K (128K256K ڴ 512 ֽڵ)
*/
struct rf_fat_mkfs_param {
    rf_int_t        sector_size;            /* Сֽ 0ΪԶȡȡ */
    rf_int_t        fat_type;               /* fatļϵͳ RF_TYPE_FAT12 RF_TYPE_FAT16 RF_TYPE_FAT32 0ΪݷԶѡ  */
    rf_int_t        fat_size;               /* fat 1 2 */
    rf_int_t        cluster_size;           /* ֽ 0ΪԶȡ */
    rf_lloff_t      volume_offset;          /* ʼƫ */
    rf_u32_t        volume_size;            /* С -1ΪԶȡ */
};
typedef struct rf_fat_mkfs_param rf_fat_mkfs_param_t, *rf_fat_mkfs_param_ref_t;

struct rf_statfs {
    rf_u16_t        f_type;                 /* ļϵͳ rfs.h ļϵͳͶ */
    rf_u16_t        f_ssize;                /* һֽ */
    rf_u32_t        f_bsize;                /* һصֽ */
    rf_u32_t        f_bfree;                /* жٸ */
    rf_u32_t        f_blocks;               /* ӵеܴ */
};
typedef struct rf_statfs rf_statfs_t, *rf_statfs_ref_t;

struct rf_stat_i {
    rf_fentry_t     entry;
    rf_u32_t        st_mode;
    rf_fsize_t      current_offset;
    rf_fsize_t      st_size;
};
typedef struct rf_stat_i rf_stat_t, *rf_stat_ref_t;

/* ΪԺչposixṹ */
struct rf_stat {
    rf_fentry_t     entry;
    rf_u32_t        st_mode;
    rf_fsize_t      current_offset;
    rf_fsize_t      st_size;
};

struct rf_file_ftime {
    rf_ftime_t      create;
    rf_ftime_t      last_write;
    rf_ftime_t      last_access;
};
typedef struct rf_file_ftime rf_file_ftime_t, *rf_file_ftime_ref_t;

struct rf_file_ltime {
    rf_tm_t         create;
    rf_tm_t         last_write;
    rf_tm_t         last_access;
};
typedef struct rf_file_ltime rf_file_ltime_t, *rf_file_ltime_ref_t;

struct rf_dirent_i {
    rf_fentry_t     entry;          /*ļƫ*/
    rf_u32_t        attribute;      /*ļ*/
    rf_int_t        type;           /*ļ ͡ansi unicode utf8 */
    rf_int_t        len;            /*ļ*/
    rf_char_t*       name;          /*ļַ*/
};
typedef struct rf_dirent_i rf_dirent_t, *rf_dirent_ref_t;

typedef struct RF_DIR RF_DIR;

struct rf_dirent {
    rf_char_t       d_name[RF_MAX_FNAME];
    rf_u32_t        attrib;
};

struct RF_DIR {
    rf_void_t*           fd;
    struct rf_dirent    result;     /* d_name null iff first time */
    rf_char_t*           name;      /* null-terminated char string */
};

struct rf_iobuf {
    rf_char_t*   _ptr;
    rf_ssize_t    _cnt;
    rf_char_t*   _base;
    rf_int_t    _flag;
    rf_int_t    _file;
    rf_int_t    _charbuf;
    rf_ssize_t    _bufsiz;
};
typedef struct rf_iobuf RF_FILE;

#endif
