/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/

#ifndef __RFS_H__
#define __RFS_H__

#include "rf_commdef.h"
#include "rf_config.h"
#include "rf_compiler.h"
#include "rf_type.h"
#include "rf_keyword.h"
#include "rf_struct.h"
#include "rf_driver.h"
#if RF_OPT_USE_STDIO
#include "rf_stdio.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define RF_O_RDONLY                 0x00    /* open for reading only */
#define RF_O_WRONLY                 0x01    /* open for writing only */
#define RF_O_RDWR                   0x02    /* open for reading and writing */
#define RF_O_APPEND                 0x04    /* writes done at eof */
#define RF_O_TRUNC                  0x08    /* open and truncate */
#define RF_O_CREAT                  0x10    /* create and open file */
#define RF_O_DIRECTORY              0x20    /* Create a directory flag */
#define RF_O_RAW_SFN                0x40    /* Open the short file names flag  */
#define RF_O_LONGNAME               0x80    /* Open the long file names flag */
#define RF_O_SECBUF                 0x100   /* sector buffer read and write files*/
#define RF_O_NOCACHE                0x200   /* file does not use the cache */
#define RF_O_ALLOC_PATH             0x400   /* rf_mount alloc path */
#define RF_O_READDIR_SFN            0x800   /* readdir returns short file names */

#define RF_SEEK_SET                 0
#define RF_SEEK_CUR                 1
#define RF_SEEK_END                 2

/* winļԶ */
#define RF_ATTR_READONLY            0X01    /* ֻ */
#define RF_ATTR_HIDDEN              0X02    /*  */
#define RF_ATTR_SYSTEM              0X04    /* ϵͳ */
#define RF_ATTR_VOLUME_ID           0X08    /*  */
#define RF_ATTR_LFN_ENTRY           0X0F    /* ļ */
#define RF_ATTR_DIRECTORY           0X10    /* Ŀ¼ */
#define RF_ATTR_ARCHIVE             0X20    /* 浵 */
#define RF_ATTR_DEVICE              0X40    /* 豸 */
#define RF_ATTR_NORMAL              0X80    /*  */
#define RF_ATTR_TEMPORARY           0X0100  /* ʱ */
#define RF_ATTR_SPARSEFILE          0X0200  /* ϡ */
#define RF_ATTR_REPARSEPOINT        0x0400  /* ؽ */
#define RF_ATTR_COMPRESSED          0x0800  /* ѹ */
#define RF_ATTR_OFFLINE             0x1000  /* ѻ */
#define RF_ATTR_NOTCONTENTINDEXED   0x2000  /* δ */
#define RF_ATTR_ENCRYPTED           0x4000  /*  */
#define RF_ATTR_INDEX               0x20000000  /* ͼ */

/* ļռ */
#define RF_NAME_WIN32               1           /* win32 ռļ */
#define RF_NAME_DOS                 2           /* dos ռļ */

/* жļǲĿ¼ */
#define RF_S_ISDIR(mode)            (((mode)&RF_ATTR_DIRECTORY))
#define RF_S_ISREG(mode)            (!((mode)&RF_ATTR_DIRECTORY))

/* ļϵͳͶ */
enum {
    RF_TYPE_FAT12 = 1,
    RF_TYPE_FAT16,
    RF_TYPE_FAT32,
    RF_TYPE_EXFAT,
    RF_TYPE_NTFS,
    RF_TYPE_EXT2,
    RF_TYPE_EXT3,
    RF_TYPE_EXT4,
};

/* ش */
#define RF_ESUCCESS             0   /* Execute successfully */
#define RF_ERROR                1   /* Unknown error */
#define RF_EINVAL               2   /* Invalid argument */
#define RF_EBADF                3   /* Bad file number */
#define RF_EBUFSMALL            4   /* Input buffer is too small */
#define RF_ENAMETOOLONG         5   /* File name is too long*/
#define RF_ENAMEENCODING        6   /* File name Encoding error */
#define RF_EXDEV                7   /* Cross-device link */
#define RF_EFBIG                8   /* File too large */

#define RF_EACCES               10  /* Permission denied */
#define RF_EPERM                11  /* Operation not permitted */
#define RF_ENOMEM               12  /* Out of memory */
#define RF_EMVOL                13  /* Too many open volume */
#define RF_EMFILE               14  /* Too many open files */

#define RF_EISDIR               20  /* Is a directory */
#define RF_ENOTDIR              21  /* Not a directory */
#define RF_ENOENT               22  /* No such file or directory */
#define RF_ENDFILE              23  /* end of file */
#define RF_ENOSPC               24  /* No space left on device */
#define RF_ENOTEMPTY            25  /* Directory not empty */

#define RF_ENOTRDY              30  /* Disk not Ready */
#define RF_EIO                  31  /* Input/output error */
#define RF_ENOFS                32  /* Invalid file system */
#define RF_EWRPRT               33  /* Write Protected error */
#define RF_EROFS                34  /* Read-only file system */

/*------------ ϵͳapi------------ */
RF_API rf_int_t rf_init(rf_void_t);
RF_API rf_int_t rf_exit(rf_void_t);

RF_API rf_version_t* rf_version(rf_void_t);
RF_API rf_int_t* rf_get_errno(rf_void_t);
RF_API rf_char_t* rf_strerror(rf_int_t errnum);
#define rf_errno (*rf_get_errno())

/* ------------ļapi---------- */
RF_API rf_int_t rf_part(rf_char_t* name, rf_part_param_t* param);
RF_API rf_int_t rf_mkfs(const rf_char_t* fs_type, const rf_char_t* device_name, const rf_void_t* ctx);
RF_API rf_int_t rf_mount(const rf_char_t* name, const rf_char_t* path,
                         const rf_char_t* fs_type, rf_int_t flags, const rf_mount_ctx_t* ctx);
RF_API rf_int_t rf_unmount(const rf_char_t* path);
RF_API rf_int_t rf_statfs(const rf_char_t* path, struct rf_statfs* buf);
RF_API rf_int_t rf_chdir(const rf_char_t* path);
RF_API rf_int_t rf_getcwd(rf_char_t* buf, rf_size_t size);

RF_API rf_int_t rf_open(const rf_char_t* file, rf_int_t flags);
RF_API rf_int_t rf_raw_open(const rf_char_t* path, rf_fentry_t* fentry, rf_int_t flags);
RF_API rf_int_t rf_close(rf_int_t fd);

RF_API rf_int_t rf_ioctl(rf_int_t fd, rf_u32_t cmd, rf_void_t* buf);
RF_API rf_int_t rf_fstat(rf_int_t fd, rf_stat_t* buf);
RF_API rf_off_t rf_lseek(rf_int_t fd, rf_off_t offset, rf_int_t whence);
RF_API rf_ssize_t rf_read(rf_int_t fd, rf_void_t* buf, rf_size_t count);

RF_API rf_int_t rf_ftruncate(rf_int_t fd, rf_off_t length);
RF_API rf_ssize_t rf_write(rf_int_t fd, const rf_void_t* buf, rf_size_t count);
RF_API rf_int_t rf_fsync(rf_int_t fd);
RF_API rf_int_t rf_fdelete(rf_int_t fd);

RF_API rf_int_t rf_rename(const rf_char_t* oldname, const rf_char_t* newname);
RF_API rf_int_t rf_unlink(const rf_char_t* pathname);

RF_API rf_int_t rf_mkdir(const rf_char_t* path);
#define  rf_rmdir rf_unlink

RF_API rf_int_t rf_readdir_ex(rf_int_t fd, const rf_char_t* pattern, struct rf_dirent_i* ptr);
#define rf_opendir_i(name) rf_open(name, RF_O_RDONLY | RF_O_DIRECTORY)
#define rf_readdir_i(fd, ptr) rf_readdir_ex( fd, "*", ptr)
#define rf_closedir_i rf_close

/*  ------------ļapi2---------- */
RF_API rf_int_t rf_stat(const rf_char_t* pathname, struct rf_stat* buf);
/* չapi ڻȡļʱ */
RF_API rf_int_t rf_get_name(rf_int_t fd, rf_char_t* buf, rf_u32_t len);
RF_API rf_int_t rf_get_long_name(rf_int_t fd, rf_wchar_t* buf, rf_int_t len);
RF_API rf_int_t rf_get_time(rf_int_t fd, rf_file_ftime_ref_t ft);
RF_API rf_int_t rf_set_time(rf_int_t fd, rf_ftime_ref_t create, rf_ftime_ref_t last_access, rf_ftime_ref_t last_write);
RF_API rf_int_t rf_get_ltime(rf_int_t fd, rf_tm_ref_t create, rf_tm_ref_t last_access, rf_tm_ref_t last_write);
RF_API rf_int_t rf_set_ltime(rf_int_t fd, rf_tm_ref_t create, rf_tm_ref_t last_access, rf_tm_ref_t last_write);
RF_API rf_int_t rf_set_attrib(rf_int_t fd, rf_u32_t attrib);
RF_API rf_int_t rf_set_file_ctx(rf_int_t fh, rf_void_ref_t ctx);
RF_API rf_int_t rf_get_file_ctx(rf_int_t fd, rf_void_ref_t* ctx);

/* ------------ļչapi---------- */
RF_API RF_DIR* rf_opendir(const rf_char_t*  name);
RF_API struct rf_dirent* rf_readdir(RF_DIR* dir);
RF_API rf_int_t rf_closedir(RF_DIR* dir);
RF_API rf_void_t rf_rewinddir(RF_DIR* dir);

RF_API rf_handle_t rf_findfirst(const rf_char_t* path, struct rf_dirent* ptr);
RF_API rf_int_t rf_findnext(rf_handle_t fh, struct rf_dirent* ptr);
RF_API rf_int_t rf_findclose(rf_handle_t fd);

RF_API rf_int_t rf_mkdir_p(const rf_char_t* path);

/*  ------------ϵͳapi------------ */
#if RF_OPT_DYNAMIC_LIB
RF_API rf_int_t rf_set_drv_func(rf_drv_ref_t drv);
#endif
RF_API rf_int_t rf_get_volume_handle_count(rf_int_t* available);
RF_API rf_int_t rf_get_file_handle_count(rf_int_t* available);
RF_API rf_int_t rf_get_fs_type(const rf_char_t* path);
RF_API const rf_char_t* rf_get_volume_dev_name(const rf_char_t* path);
RF_API rf_bool_t rf_get_volume_mount_status(const rf_char_t* path);
RF_API rf_void_t* rf_get_volume_sector_buf(const rf_char_t* path);
RF_API rf_int_t rf_get_volume_sector_size(const rf_char_t* path);

/*  -----------תapi------------ */
RF_API rf_int_t rf_iconv(const rf_void_t* in_str, rf_int_t in_cp, rf_void_t* out_str, rf_int_t out_len, rf_int_t out_cp);

#ifdef __cplusplus
}
#endif

#endif
