/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_NTFS_FILE_H__
#define __RF_NTFS_FILE_H__

struct _rf_ntfs_file_info {
    /* ļ: MFT ˲ #ռ# ȡƫ */
    rf_u16_t    data_body_offset;
    /* ļ: MFT Ŀ¼: ļMFT */
    rf_u64_t    mft_data_sector;
    /* ļ: ǰĳ */
    rf_u64_t    length;
};
typedef struct _rf_ntfs_file_info rf_ntfs_file_info;

struct _rf_ntfs_dir_info {
    /* öidxɱ־λ */
    rf_u8_t     enum_idx_root_flags;
    /* һ¼ռü λ */
    rf_u8_t     clust_per_index_record_bits;
    /* ¼¼µƫ */
    rf_u16_t    enum_record_sector_offset;
    /* Ŀ¼ȡƫ */
    rf_u16_t    index_alloc_body_offset;
    /* ¼öʼ */
    rf_u32_t    enum_record_last_cluster;

    /* ȡVCN */
    rf_u32_t    enum_vcn_index;
    /* öļڵǰ [] ƫ */
    rf_u32_t    enum_last_offset;
    /* öļ [] ƫ */
    rf_u32_t    enum_clust_max_offset;

    /* Ŀ¼: ļMFT */
    rf_u64_t    index_root_sector;
    /* Ŀ¼: MFT */
    rf_u64_t    index_alloc_sector;
    /* Ŀ¼: λͼMFT */
    rf_u64_t    index_bitmap_sector;
};
typedef struct _rf_ntfs_dir_info rf_ntfs_dir_info;

struct _rf_ntfs_file_t {
    /* 򿪻߹ر ־λ */
    rf_u8_t             open_flag: 1;
    /* 0ֻ 1д 2дȨ */
    rf_u8_t             access_flag: 2;
    /* 1 ļ־ */
    rf_u8_t             sector_buf_flag: 1;
    /* 1 ǰΪĿ¼ */
    rf_u8_t             root_dir_flag: 1;
    /* Ŀ¼ڷ */
    rf_ntfs_volume_ref_t vol;
    /* ļʹλ */
#if RF_OPT_USE_FILE_CTX
    /* ļ */
    rf_void_ref_t       ctx;
#endif
    /* ̬malloc fileʱ ʹ */
#if (RF_OPT_USE_MALLOC == 1)
    list_t list;
#endif

    /* дǷѼ ־λ */
    rf_u8_t             gather_cluester_bit: 1;
    /* MFTб Ƿʱ */
    rf_u8_t             list_run_bit : 1;
    /* MFT Ƿʱ Ŀ¼: Ƿ */
    rf_u8_t             data_run_bit : 1;
    /* ļ */
    rf_u32_t            attrib;
    /* ļƫ */
    rf_fentry_t         entry;
    /* бʱ ˲ #ռ# ȡбƫ */
    rf_u16_t            list_body_offset;
    /* б Ϊǰƫ */
    rf_u64_t            list_search_offset;
    /* дƫ */
    rf_u64_t            current_offset;
    /* ļؽṹ */
    rf_chain_idx_t      clust;

    union {
        rf_ntfs_file_info fi;
        rf_ntfs_dir_info di;
    } u;
};
typedef struct _rf_ntfs_file_t rf_ntfs_file_t, *rf_ntfs_file_ref_t;

rf_int_t ntfs_file_open(rf_ntfs_file_ref_t file, const rf_tchar_t* name, rf_int_t flags);
rf_int_t ntfs_file_raw_open(rf_ntfs_file_ref_t file, rf_fentry_t* fentry, rf_u32_t open_flag);
rf_int_t ntfs_file_lseek(rf_ntfs_file_ref_t file, rf_u64_t offset, rf_int_t origin);
rf_int_t ntfs_file_read(rf_ntfs_file_ref_t file, rf_void_t* buf, rf_u32_t len, rf_u32_t* plen);
rf_int_t ntfs_file_close(rf_ntfs_file_ref_t file);
rf_int_t ntfs_file_get_stat(rf_ntfs_file_ref_t file, rf_stat_ref_t st);
rf_int_t ntfs_file_get_ftime(rf_ntfs_file_ref_t file, rf_file_ftime_ref_t ft);
rf_int_t ntfs_file_read_dir(rf_ntfs_file_ref_t file, const rf_char_t* pattern, rf_dirent_ref_t pfd);
rf_int_t ntfs_file_get_name(rf_ntfs_file_ref_t file, rf_char_t* buf, rf_u32_t len);
rf_int_t ntfs_file_get_long_name(rf_ntfs_file_ref_t file, rf_wchar_t* buf, rf_u32_t len);

rf_bool_t ntfs_check_file_access(rf_ntfs_file_ref_t file);

#endif
