/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_NTFS_VOLUME_H__
#define __RF_NTFS_VOLUME_H__

#define MAX_MFT_BLOCK               3

struct _rf_ntfs_volume_t {
    /* ļϵͳ */
    rf_u8_t fs_type: 4;
    /* 򿪱־ */
    rf_u8_t open_flag: 1;
    /*  ģʽ 0 1д */
    rf_u8_t access_flag: 1;
#if RF_OPT_FS_READONLY == 0
    /* µǰ־λ */
    rf_u8_t write_flag: 1;
#endif
#if RF_OPT_USE_MALLOC
    list_t list;
#endif
#if RF_OPT_USE_VOLUME_CTX
    rf_void_t* ctx;
#endif

    /* 豸ָ */
    rf_handle_t dev_handle;
#if RF_OPT_USE_MALLOC
    /*  */
    rf_u8_t* sector_buffer;
#else
    rf_u8_t sector_buffer[RF_OPT_MAX_SECTOR_SIZE];
#endif

    /* ǰĿ¼ ڵ */
    rf_fentry_t current_dir_entry;
    /* Ŀ¼ ڵ */
    rf_fentry_t root_dir_entry;

    /* ֽڵλ λ */
    rf_u8_t     sector_size_bits;
    /* صĴСλ λ */
    rf_u8_t     cluster_size_bits;
    /* һؼλ λ */
    rf_u8_t     sectors_per_cluster_bits;
    /* һ  */
    rf_u8_t     sectors_per_cluster;
    /* С */
    rf_u16_t    sector_size;
    /* صĴСֽ */
    rf_u32_t    cluster_size;
    /* ǰٸ */
    rf_s32_t    total_clusters;

    /* һMFTռ */
    rf_u8_t     mft_sizeof_sector;
    rf_u8_t     mft_record_size_bits;
    rf_u8_t     index_record_size_bits;
    rf_u8_t     mft_zone_multiplier;

    /* MFTʼLCN */
    rf_s32_t    mft_lcn;
    /* MFTݾʼLCN */
    rf_s32_t    mft_mirr_lcn;

    rf_s32_t    mft_zone_pos;
    rf_s32_t    mft_zone_start;
    rf_s32_t    mft_zone_end;
    rf_s32_t    data1_zone_pos;
    rf_s32_t    data2_zone_pos;

    rf_s32_t    mft_clusters_per_record;
    rf_u32_t    mft_record_size;
    rf_s32_t    index_clusters_per_record;
    rf_u32_t    index_record_size;

    rf_u32_t    mft_offet[MAX_MFT_BLOCK], mft_size[MAX_MFT_BLOCK];
};
typedef struct _rf_ntfs_volume_t rf_ntfs_volume_t, *rf_ntfs_volume_ref_t;

rf_int_t ntfs_volume_open(rf_ntfs_volume_ref_t vol, rf_mount_param_t* param);
rf_int_t ntfs_volume_close(rf_ntfs_volume_ref_t vol);
rf_int_t ntfs_volume_stat(rf_ntfs_volume_ref_t vol, rf_statfs_ref_t pvs);
rf_int_t ntfs_volume_get_label(rf_ntfs_volume_ref_t vol, const rf_char_t* buf, rf_int_t len);
rf_int_t ntfs_volume_set_label(rf_ntfs_volume_ref_t vol, const rf_char_t* buf);
rf_int_t ntfs_volume_set_current_dir(rf_ntfs_volume_ref_t vol, rf_fentry_t* entry);

#endif
