/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "rfs.h"

struct _disk_context_t {
    rf_int_t flag;
    rf_lloff_t volume_offset;
};
typedef struct _disk_context_t rf_disk_context_t, *rf_disk_context_ref_t;

#if RF_OPT_USE_MALLOC == 0
static rf_disk_context_t g_disk_ctx[RF_OPT_MAX_OPEN_DISK];
#endif

rf_int_t rf_disk_init(void)
{
#if RF_OPT_USE_MALLOC == 0
    int i;
    for (i = 0; i < RF_OPT_MAX_OPEN_DISK; i++)
        g_disk_ctx[i].flag = 0;
#endif
    return 0;
}

static rf_disk_context_ref_t disk_context_alloc(void)
{
#if RF_OPT_USE_MALLOC == 0
    int i;

    for (i = 0; i < RF_OPT_MAX_OPEN_DISK; i++)
        if (g_disk_ctx[i].flag == 0)
            return &g_disk_ctx[i];
    return NULL;
#else
    return malloc(sizeof(rf_disk_context_t));
#endif
}

static rf_int_t disk_context_free(rf_disk_context_ref_t dc)
{
#if RF_OPT_USE_MALLOC == 0
    dc->flag = 0;
#else
    free(dc);
#endif
    return 0;
}


/*
򿪴豸


device_name [in]    rf_mount ص豸
flags       [in]    RF_O_RDONLYֻ  RF_O_RDWR д
ctx         [in]    rf_mount_ctx_tṩز
hdev        [out]   豸 Ӧľ
עú rf_mount 

ֵ
RF_ESUCCESS ɹ
RF_ENOTRDY  û׼û
RF_EIO      ̴
RF_ENOMEM   豸ڴʧ
RF_EINVAL   Ч
RF_ERROR    δ֪
*/
rf_int_t rf_disk_open(const rf_char_t* device_name, rf_int_t flags, rf_void_t* ctx, rf_handle_t* hdev)
{
    rf_mount_ctx_ref_t m_ctx = (rf_mount_ctx_ref_t)ctx;
    rf_disk_context_ref_t disk_ctx;

    /* õײ㺯 ʼ豸 */
    disk_ctx = disk_context_alloc();
    if (disk_ctx == NULL)
        return RF_ENOMEM;
    disk_ctx->volume_offset = m_ctx->volume_offset;
    *hdev = disk_ctx;
    return RF_ESUCCESS;
}

/*
رմ豸


hdev        [in]    Ϊrf_disk_open صľ
ע:  rf_unmount 

ֵ
RF_ESUCCESS ɹ
RF_EINVAL   Ч
RF_ERROR    δ֪
*/
rf_int_t rf_disk_close(rf_handle_t hdev)
{
    return disk_context_free((rf_disk_context_ref_t)hdev);
}

/*
豸ioƺ


hdev    [in]        Ϊrf_disk_open صľ
cmd     [in]        rf_driver.h ͷļ
buf     [in out]    cmdݴŻ
len     [in]        buf

ֵ
RF_ESUCCESS ɹ
RF_ENOTRDY  û׼û
RF_EIO      ̴
RF_EINVAL   Ч
RF_ERROR    δ֪
*/
rf_int_t rf_disk_ioctl(rf_handle_t hdev, rf_int_t cmd, rf_void_t* buf, rf_int_t len)
{
    switch (cmd) {
    case RF_CMD_GET_SECTOR_SIZE:
        if (len != sizeof(rf_u32_t))
            return RF_EINVAL;
        /* ش ֽ */
        *((rf_u32_t*)buf) = 512;
        return RF_ESUCCESS;

    case RF_CMD_GET_SECTOR_COUNT:
        /* ش */
        return RF_EINVAL;

    case RF_CMD_GET_BLOCK_SIZE:
        /* ȡС ʽʹ */
        if (len != sizeof(rf_u32_t))
            return RF_EINVAL;
        *((rf_u32_t*)buf) = 4096;
        return RF_ESUCCESS;

    case RF_CMD_CTRL_ERASE_SECTOR:
        /* 豸 ʽʹ */
        return RF_ESUCCESS;

    default:
        return RF_EINVAL;
    }
}

/*
豸


hdev        [in]    Ϊrf_disk_openصľ
sec_offset  [in]    Ϊƫ
sec_count   [in]    Ϊȡ
buf         [out]   ΪݴŻ

ֵ
RF_ESUCCESS ɹ
RF_ENOTRDY  û׼û
RF_EINVAL   Ч
RF_EIO      ̴
RF_ERROR    δ֪
*/
rf_int_t rf_disk_read(rf_handle_t hdev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    rf_disk_context_ref_t disk_ctx = (rf_disk_context_ref_t)hdev;

    /* ȡƫ sec_offset Ҫ disk_ctx->volume_offset */
    return RF_ESUCCESS;
}

#if RF_OPT_FS_READONLY == 0
/*
д


hdev        [in]    Ϊrf_disk_openصľ
sec_offset  [in]    Ϊƫ
sec_count   [in]    Ϊȡ
buf         [in]    ΪݴŻ

ֵ
RF_ESUCCESS ɹ
RF_EWRPRT   д
RF_ENOTRDY  û׼û
RF_EIO      ̴
RF_EINVAL   Ч
RF_ERROR    δ֪
*/
rf_int_t rf_disk_write(rf_handle_t hdev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    rf_disk_context_ref_t disk_ctx = (rf_disk_context_ref_t)hdev;

    /* дƫ sec_offset Ҫ disk_ctx->volume_offset */
    return RF_EIO;
}
#endif

/*
fatļʱ ʱ Ҫô˺ رʱ


tm          [in]    ʱ䷵ؽṹ

ֵ
RF_ESUCCESS ɹ
RF_ERROR ֧ʱ ļȲʹʱ
*/
rf_int_t rf_getlocaltime(rf_tm_ref_t tm)
{
    /* ȡǰʱʵ */
    tm->tm_sec = 1;
    tm->tm_min = 2;
    tm->tm_hour = 3;

    tm->tm_mday = 1;
    tm->tm_mon = 9;
    tm->tm_year = 2015;
    return RF_ESUCCESS;
}
