/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "../common/rf_common.h"
#include "rf_fs.h"
#include "rf_file.h"
#if RF_OPT_USE_FAT
#include "../fat/fat_api.h"
#endif
#if RF_OPT_USE_EXT2
#include "../ext2/ext2_api.h"
#endif
#if RF_OPT_USE_NTFS
#include "../ntfs/ntfs_api.h"
#endif

#if RF_OPT_USE_GET_HANDLE_COUNT
rf_int_t rf_get_volume_handle_count(rf_int_t* available)
{
    rf_int_t use_count = 0, available_count = 0, tmp;

#if RF_OPT_USE_FAT
    use_count += fat_get_volume_handle_count(&tmp);
    available_count += tmp;
#endif

#if RF_OPT_USE_EXT2
    use_count += ext2_get_volume_handle_count(&tmp);
    available_count += tmp;
#endif

#if RF_OPT_USE_NTFS
    use_count += ntfs_get_volume_handle_count(&tmp);
    available_count += tmp;
#endif

    if (available)
        *available = available_count;
    return use_count;
}

rf_int_t rf_get_file_handle_count(rf_int_t* available)
{
    rf_int_t use_count = 0, available_count = 0, tmp;

#if RF_OPT_USE_FAT
    use_count += fat_get_file_handle_count(&tmp);
    available_count += tmp;
#endif

#if RF_OPT_USE_EXT2
    use_count += ext2_get_file_handle_count(&tmp);
    available_count += tmp;
#endif

#if RF_OPT_USE_NTFS
    use_count += ntfs_get_file_handle_count(&tmp);
    available_count += tmp;
#endif

    if (available)
        *available = available_count;
    return use_count;
}
#endif

rf_int_t rf_get_fs_type(const rf_char_t* path)
{
    rf_statfs_t st;
    if (rf_statfs(path, &st))
        return -1;
    return st.f_type;
}

rf_bool_t rf_get_volume_mount_status(const rf_char_t* path)
{
    rf_vol_ctx_t* fs;

    fs = find_fs_ctx(path);
    if (fs)
        return fs->open_flag;
    return 0;
}

#if RF_OPT_FS_SAVE_DEV_NAME
const rf_char_t* rf_get_volume_dev_name(const rf_char_t* path)
{
    rf_vol_ctx_t* fs;

    fs = find_fs_ctx(path);
    if (fs == NULL) {
        rf_set_errno(RF_EINVAL);
        return NULL;
    }
    return fs->device_name;
}
#endif
