#include <stdio.h>
#include <windows.h>
#include <winioctl.h>
#include "rfs.h"
#include "win32_drv.h"
#include "liblog.h"

rf_int_t file_drv_open(rf_handle_t dev, const rf_char_t* dev_name, rf_int_t flags)
{
    DWORD dwFlagsAndAttributes;
    rf_handle_t handle;
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;
    LARGE_INTEGER file_size;

    dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL;
    if (flags & RF_O_NOCACHE)
        dwFlagsAndAttributes |= FILE_FLAG_NO_BUFFERING;

    if ((flags & 0x3) == 2) {
        /* дļ  ֻһд */
        handle = (rf_handle_t)CreateFile(dev_name, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,
                                         NULL, OPEN_EXISTING, dwFlagsAndAttributes, NULL);

    } else {
        /*  ̶д */
        handle = (rf_handle_t)CreateFile(dev_name, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                                         NULL, OPEN_EXISTING, dwFlagsAndAttributes, NULL);
    }
    if (handle == INVALID_HANDLE_VALUE) {
        log_perror("CreateFile:%s error: ", dev_name);
        return RF_EIO;
    }
    if (!GetFileSizeEx(handle, &file_size)) {
        CloseHandle(handle);
        return RF_EIO;
    }

    ctx->handle = handle;
    // ļ ĬС
    ctx->sector_size = 512;
    ctx->file_size = (rf_lloff_t)file_size.QuadPart;
    return 0;
}

rf_int_t file_drv_close(rf_handle_t dev)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;

    return CloseHandle(ctx->handle) ? 0 : 1;
}

rf_int_t file_drv_ioctl(rf_handle_t dev, rf_u32_t cmd, rf_void_t* buf, rf_u32_t len)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;

    switch (cmd) {
    case RF_CMD_GET_SECTOR_SIZE:
        if (len != sizeof(rf_u32_t))
            return RF_EINVAL;
        *((rf_u32_t*)buf) = ctx->sector_size;
        return 0;

    case RF_CMD_GET_SECTOR_COUNT:
        if (len != sizeof(rf_lloff_t))
            return RF_EINVAL;
        *((rf_lloff_t*)buf) = ctx->file_size / ctx->sector_size;
        return 0;

    case RF_CMD_CTRL_SYNC:
        return FlushFileBuffers(ctx->handle) ? 0 : RF_EIO;

    default:
        return RF_EINVAL;
    }
}

rf_int_t file_drv_read(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;
    rf_u32_t len;
    LARGE_INTEGER li;

    li.QuadPart = (ctx->volume_offset + (rf_u64_t)sector_offset) * ctx->sector_size;
    if (!SetFilePointerEx(ctx->handle, li, NULL, FILE_BEGIN)) {
        log_perror("SetFilePointerEx: ");
        return RF_EIO;
    }
    if (!ReadFile(ctx->handle, buf, (DWORD)(sector_count * ctx->sector_size), (LPDWORD)&len, NULL)) {
        log_perror("ReadFile: ");
        return RF_EIO;
    }
    return 0;
}

rf_int_t file_drv_write(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;
    rf_u32_t len;
    LARGE_INTEGER li;

    li.QuadPart = (ctx->volume_offset + (rf_u64_t)sector_offset) * ctx->sector_size;
    if (!SetFilePointerEx(ctx->handle, li, NULL, FILE_BEGIN)) {
        log_perror("SetFilePointerEx: ");
        return RF_EIO;
    }
    if (!WriteFile(ctx->handle, buf, (DWORD)(sector_count * ctx->sector_size), (LPDWORD)&len, NULL)) {
        log_perror("WriteFile: ");
        return RF_EIO;
    }
    return 0;
}
