#ifndef __RF_LOGGING_H__
#define __RF_LOGGING_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>

#ifndef RF_LOG_TAG
#define RF_LOG_TAG              "rf.librfs"
#endif

#define LOG_CMD_SET_LEVELS      1
#define LOG_CMD_SET_FLAGS       2
#define LOG_CMD_SET_STR_LEVELS  3
#define LOG_CMD_SET_STR_FLAGS   4

#define LOG_CMD_SET_STDOUT_FUNC 5
#define LOG_CMD_SET_STDERR_FUNC 6
#define LOG_CMD_SET_FILE        7
#define LOG_CMD_SET_FILE_SIZE   8
#define LOG_CMD_SET_NET         9

#define LOG_CMD_ENABLE_SH       10
#define LOG_CMD_ENABLE_WIN      11
#define LOG_CMD_ENABLE_FILE     12
#define LOG_CMD_ENABLE_NET      13

#define LOG_FLAG_TIME           (1 << 0)
#define LOG_FLAG_PREFIX         (1 << 1)
#define LOG_FLAG_FILELINE       (1 << 2)
#define LOG_FLAG_FUNCTION       (1 << 3)
#define LOG_FLAG_ONLYNAME       (1 << 4)
#define LOG_FLAG_COLOUR         (1 << 5)
#define LOG_FLAG_TID            (1 << 6)
#define LOG_OPT_SHOW_ERRNO      0x100

#define LOG_EMERF               (0)
#define LOG_ALERT               (1)
#define LOG_CRIT                (2)
#define LOG_ERROR               (3)
#define LOG_WARNING             (4)
#define LOG_NOTICE              (5)
#define LOG_INFO                (6)
#define LOG_DEBUG               (7)
#define LOG_VERBOSE             (8)

#ifndef RF_NOLOG
#define log_verb(FORMAT, ...)       log_trace( LOG_VERBOSE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_debug(FORMAT, ...)      log_trace( LOG_DEBUG, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_info(FORMAT, ...)       log_trace( LOG_INFO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_notice(FORMAT, ...)     log_trace( LOG_NOTICE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_warn(FORMAT, ...)       log_trace( LOG_WARNING, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_error(FORMAT, ...)      log_trace( LOG_ERROR, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_perror(FORMAT, ...)     log_trace( LOG_ERROR|LOG_OPT_SHOW_ERRNO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define log_crit(FORMAT, ...)       log_trace( LOG_CRIT, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)

#define log_verbv(FORMAT, VA)       log_trace( LOG_VERBOSE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_debugv(FORMAT, VA)      log_trace( LOG_DEBUG, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_infov(FORMAT, VA)       log_trace( LOG_INFO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_noticev(FORMAT, VA)     log_trace( LOG_NOTICE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_warnv(FORMAT, VA)       log_trace( LOG_WARNING, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_errorv(FORMAT, VA)      log_trace( LOG_ERROR, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_perrorv(FORMAT, VA)     log_trace( LOG_ERROR|LOG_OPT_SHOW_ERRNO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)
#define log_critv(FORMAT, VA)       log_trace( LOG_CRIT, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, VA)

#define logw_verb(FORMAT, ...)      log_wtrace( LOG_VERBOSE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_debug(FORMAT, ...)     log_wtrace( LOG_DEBUG, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_info(FORMAT, ...)      log_wtrace( LOG_INFO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_notice(FORMAT, ...)    log_wtrace( LOG_NOTICE, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_warn(FORMAT, ...)      log_wtrace( LOG_WARNING, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_error(FORMAT, ...)     log_wtrace( LOG_ERROR, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_perror(FORMAT, ...)    log_wtrace( LOG_ERROR|LOG_OPT_SHOW_ERRNO, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)
#define logw_crit(FORMAT, ...)      log_wtrace( LOG_CRIT, RF_LOG_TAG, __FILE__, __LINE__, __FUNCTION__, FORMAT, ##__VA_ARGS__)

#else

#define log_debug(FORMAT, ... )
#define log_info(FORMAT, ...  )
#define log_notice(FORMAT, ...  )
#define log_warn(FORMAT, ...  )
#define log_error(FORMAT, ...  )
#define log_perror(FORMAT, ...)
#define log_werror(FORMAT, ...)
#define log_crit(FORMAT, ... )

#define logw_debug(FORMAT, ... )
#define logw_info(FORMAT, ...  )
#define logw_notice(FORMAT, ...  )
#define logw_warn(FORMAT, ...  )
#define logw_error(FORMAT, ...  )
#define logw_perror(FORMAT, ...)
#define logw_werror(FORMAT, ...)
#define logw_crit(FORMAT, ... )
#endif

#define log_printf(FORMAT, ...)      log_printf_imp( LOG_INFO, RF_LOG_TAG, FORMAT, ##__VA_ARGS__)
#define log_eprintf(FORMAT, ...)     log_printf_imp( LOG_ERROR, RF_LOG_TAG, FORMAT, ##__VA_ARGS__)
#define log_hexdump(ptr, len)        log_hexdump_imp( LOG_INFO, RF_LOG_TAG, ptr, len)

#if defined(_MSC_VER)
#   define __log_export__           __declspec(dllexport)
#else
#   define __log_export__
#endif

typedef void (log_handler)(const char* data, int len);

#define LOG_API extern
LOG_API int log_ioctl(int cmd, void* buf, int len);
LOG_API int log_trace(int level, const char* tag,
                      const char* file, int line, const char* func, const char* format, ...);
LOG_API int log_vtrace(int level, const char* tag,
                       const char* file, int line, const char* func, const char* format, va_list args);
#if defined(__cplusplus) && defined(_UNICODE)
LOG_API int log_wtrace(int level, const char* tag,
                       const char* file, int line, const char* func, const wchar_t* format, ...);
#endif
LOG_API int log_printf_imp(int level, const char* tag, const char* format, ...);
LOG_API int log_hexdump_imp(int level, const char* tag, const void *ptr, int len );
LOG_API int log_write(int level, const char* tag, const char* buf, int len);

LOG_API int log_get_level(const char* level);
LOG_API int log_main(int argc, char* argv[]);
LOG_API int log_init( void );

#ifdef __cplusplus
}
#endif

#endif

