/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "rfs.h"
#include "win32_drv.h"
#include <Windows.h>

rf_int_t rf_disk_init(void)
{
    return 0;
}

rf_int_t rf_disk_open(const rf_char_t* device_name, rf_int_t flags, rf_void_t* ctx, rf_handle_t* hdev)
{
    return win_open(device_name, flags, ctx, hdev);
}

rf_int_t rf_disk_close(rf_handle_t hdev)
{
    return win_close(hdev);
}

rf_int_t rf_disk_ioctl(rf_handle_t dev, rf_int_t cmd, rf_void_t* buf, rf_int_t len)
{
    return win_ioctl(dev, cmd, buf, len);
}

rf_int_t rf_disk_read(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    return win_read(dev, sec_offset, sec_count, buf);
}

#if RF_OPT_FS_READONLY == 0
rf_int_t rf_disk_write(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    return win_write(dev, sec_offset, sec_count, buf);
}
#endif

rf_int_t rf_getlocaltime(rf_tm_ref_t tm)
{
    SYSTEMTIME sys;

    if (tm == NULL)
        return -1;
    GetLocalTime(&sys);

    tm->tm_sec = sys.wSecond;
    tm->tm_min = sys.wMinute;
    tm->tm_hour = sys.wHour;

    tm->tm_mday = sys.wDay;
    tm->tm_mon = sys.wMonth;
    tm->tm_year = sys.wYear;

    return 0;
}
